import { Body, Controller, Get, Query, Put, Delete, Post } from '@nestjs/common';
import { ApiOperation, ApiTags } from '@nestjs/swagger';
import { ApiSecurityAuth } from '~/common/decorators/swagger.decorator';
import { ApiResult } from '~/common/decorators/api-result.decorator';
import { IdParam } from '~/common/decorators/id-param.decorator';
import { Perm, definePermission } from '~/modules/auth/decorators/permission.decorator';
import {
  SaleQuotationGroupDto,
  SaleQuotationGroupQueryDto,
  SaleQuotationGroupUpdateDto
} from './sale_quotation_group.dto';
import { SaleQuotationGroupService } from './sale_quotation_group.service';
export const permissions = definePermission('sale_quotation:sale_quotation_group', {
  LIST: 'list',
  CREATE: 'create',
  READ: 'read',
  UPDATE: 'update',
  DELETE: 'delete'
} as const);

@ApiTags('SaleQuotationGroup - 报价分组')
@ApiSecurityAuth()
@Controller('sale_quotation_group')
export class SaleQuotationGroupController {
  constructor(private saleQuotationGroupService: SaleQuotationGroupService) {}

  @Get()
  @ApiOperation({ summary: '分页获取报价分组列表' })
  @ApiResult({ type: [SaleQuotationGroupDto], isPage: true })
  @Perm(permissions.LIST)
  async list(@Query() dto: SaleQuotationGroupQueryDto) {
    return this.saleQuotationGroupService.findAll(dto);
  }

  @Get(':id')
  @ApiOperation({ summary: '获取报价分组信息' })
  @ApiResult({ type: SaleQuotationGroupDto })
  @Perm(permissions.READ)
  async info(@IdParam() id: number) {
    return this.saleQuotationGroupService.info(id);
  }

  @Post()
  @ApiOperation({ summary: '新增报价分组' })
  @Perm(permissions.CREATE)
  async create(@Body() dto: SaleQuotationGroupDto): Promise<void> {
    await this.saleQuotationGroupService.create(dto);
  }

  @Put(':id')
  @ApiOperation({ summary: '更新报价分组' })
  @Perm(permissions.UPDATE)
  async update(@IdParam() id: number, @Body() dto: SaleQuotationGroupUpdateDto): Promise<void> {
    await this.saleQuotationGroupService.update(id, dto);
  }

  @Delete(':id')
  @ApiOperation({ summary: '删除报价分组' })
  @Perm(permissions.DELETE)
  async delete(@IdParam() id: number): Promise<void> {
    await this.saleQuotationGroupService.delete(id);
  }
}
