import { Controller, Get, Query, Res } from '@nestjs/common';
import { ApiOperation, ApiTags } from '@nestjs/swagger';
import { ApiSecurityAuth } from '~/common/decorators/swagger.decorator';
import { Perm, definePermission } from '~/modules/auth/decorators/permission.decorator';
import { FastifyReply } from 'fastify';
import { SaleQuotationService } from './sale_quotation.service';
import { IdParam } from '~/common/decorators/id-param.decorator';
export const permissions = definePermission('sale_quotation:sale_quotation', {
  EXPORT: 'export'
} as const);

@ApiTags('SaleQuotation - 报价模块')
@ApiSecurityAuth()
@Controller('sale_quotation')
export class SaleQuotationController {
  constructor(private saleQuotationService: SaleQuotationService) {}

  @Get('export/:id')
  @ApiOperation({ summary: '导出报价配置明细' })
  @Perm(permissions.EXPORT)
  async export(@IdParam() id: number, @Res() res: FastifyReply): Promise<void> {
    await this.saleQuotationService.export(id, res);
  }
}
