import { Injectable } from '@nestjs/common';
import { InjectEntityManager, InjectRepository } from '@nestjs/typeorm';
import { EntityManager, Repository } from 'typeorm';
import { SaleQuotationTemplateEntity } from './template/sale_quotation_template.entity';
import { FastifyReply } from 'fastify';
import * as ExcelJS from 'exceljs';
import { SaleQuotationGroupEntity } from './group/sale_quotation_group.entity';
@Injectable()
export class SaleQuotationService {
  constructor(
    @InjectEntityManager() private entityManager: EntityManager,
    @InjectRepository(SaleQuotationTemplateEntity)
    private saleQuotationTemplateRepository: Repository<SaleQuotationTemplateEntity>
  ) {}

  /**
   * 导出报价配置明细
   */
  async export(templateId: number, res: FastifyReply): Promise<void> {
    const ROW_HEIGHT = 20;
    const HEADER_FONT_SIZE = 18;
    const workbook = new ExcelJS.Workbook();
    let data = await this.saleQuotationTemplateRepository.findOneBy({ id: templateId });
    const template: JSON = data.template;
    const sheet = workbook.addWorksheet(data.name);
    if (template != null) {
      sheet.mergeCells('A1:H1');
      // 设置标题
      sheet.getCell('A1').value = '电液控部分配置明细';
      // 设置副标题
      sheet.mergeCells('A2:F2');
      sheet.getCell('A2').value = '支架电液控系统配置明细（中间131架+过渡架4架）';
      sheet.getCell(`G2`).value = '';
      sheet.getCell(`H2`).value = '';
      const groups = template['data'] as SaleQuotationGroupEntity[];
      const headers = [
        '过渡',
        '名称',
        '规格、型号及说明',
        '单位',
        '数量',
        '备     注',
        '单价',
        '总价'
      ];

      sheet.addRow(headers);
      for (let i = 1; i <= 8; i++) {
        sheet.getCell(`${String.fromCharCode(64 + i)}1`).style.font = { bold: true };
        sheet.getCell(`${String.fromCharCode(64 + i)}2`).style.font = { bold: true };
        sheet.getCell(`${String.fromCharCode(64 + i)}3`).style.font = { bold: true };
      }

      let rowIndex = 3;
      for (let groupIndex = 0; groupIndex < groups.length; groupIndex++) {
        rowIndex++;
        const group = groups[groupIndex];
        sheet.mergeCells(`A${rowIndex}:F${rowIndex}`);
        sheet.getCell(`A${rowIndex}`).value = group.name;
        sheet.getCell(`A${rowIndex}`).style.font = { bold: true };
        sheet.getCell(`G${rowIndex}`).value = '';
        sheet.getCell(`H${rowIndex}`).value = '';
        for (let componentIndex = 0; componentIndex < group.items.length; componentIndex++) {
          const item = group.items[componentIndex];
          rowIndex++;
          sheet.addRow([
            `${componentIndex + 1}`,
            item.name ?? '',
            item.componentSpecification ?? '',
            item.unit ?? '',
            item.quantity ?? '',
            item.remark ?? '',
            item.unitPrice ?? '',
            item.amount ?? ''
          ]);
        }
      }
      sheet.getCell(`I${rowIndex - 1}`).value = '总价';
      sheet.getCell(`I${rowIndex - 1}`).style.font = { bold: true };
      sheet.getCell(`I${rowIndex}`).value = template['totalPrice'];
    }
    sheet.eachRow((row, index) => {
      if (index >= 0) {
        row.alignment = { vertical: 'middle', horizontal: 'center' };
        row.height = ROW_HEIGHT;
        row.eachCell(cell => {
          cell.border = {
            top: { style: 'thin' },
            left: { style: 'thin' },
            bottom: { style: 'thin' },
            right: { style: 'thin' }
          };
        });
      }
    });
    const columnWidthMap = { A: 8, B: 30, C: 25, F: 20 };
    sheet.columns.forEach((column, index: number) => {
      column.width = columnWidthMap[String.fromCharCode(65 + index)] ?? 10; // Minimum width of 10
    });

    //读取buffer进行传输
    const buffer = await workbook.xlsx.writeBuffer();
    res
      .header('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet')
      .header(
        'Content-Disposition',
        `attachment; filename="${encodeURIComponent('导出_excel' + new Date().getTime() + '.xls')}"`
      )
      .send(buffer);
  }
}
