import { Body, Controller, Get, Query, Put, Delete, Post } from '@nestjs/common';
import { ApiOperation, ApiTags } from '@nestjs/swagger';
import { ApiSecurityAuth } from '~/common/decorators/swagger.decorator';
import { ApiResult } from '~/common/decorators/api-result.decorator';
import { IdParam } from '~/common/decorators/id-param.decorator';
import { Perm, definePermission } from '~/modules/auth/decorators/permission.decorator';
import {
  SaleQuotationTemplateDto,
  SaleQuotationTemplateQueryDto,
  SaleQuotationTemplateUpdateDto
} from './sale_quotation_template.dto';
import { SaleQuotationTemplateService } from './sale_quotation_template.service';
export const permissions = definePermission('sale_quotation:sale_quotation_template', {
  LIST: 'list',
  CREATE: 'create',
  READ: 'read',
  UPDATE: 'update',
  DELETE: 'delete'
} as const);

@ApiTags('SaleQuotationTemplate - 报价模板')
@ApiSecurityAuth()
@Controller('sale_quotation_template')
export class SaleQuotationTemplateController {
  constructor(private saleQuotationTemplateService: SaleQuotationTemplateService) {}

  @Get()
  @ApiOperation({ summary: '分页获取报价模板列表' })
  @ApiResult({ type: [SaleQuotationTemplateDto], isPage: true })
  @Perm(permissions.LIST)
  async list(@Query() dto: SaleQuotationTemplateQueryDto) {
    return this.saleQuotationTemplateService.findAll(dto);
  }

  @Get(':id')
  @ApiOperation({ summary: '获取报价模板信息' })
  @ApiResult({ type: SaleQuotationTemplateDto })
  @Perm(permissions.READ)
  async info(@IdParam() id: number) {
    return this.saleQuotationTemplateService.info(id);
  }

  @Post()
  @ApiOperation({ summary: '新增报价模板' })
  @Perm(permissions.CREATE)
  async create(@Body() dto: SaleQuotationTemplateDto): Promise<void> {
    await this.saleQuotationTemplateService.create(dto);
  }

  @Put(':id')
  @ApiOperation({ summary: '更新报价模板' })
  @Perm(permissions.UPDATE)
  async update(@IdParam() id: number, @Body() dto: SaleQuotationTemplateUpdateDto): Promise<void> {
    await this.saleQuotationTemplateService.update(id, dto);
  }

  @Delete(':id')
  @ApiOperation({ summary: '删除报价模板' })
  @Perm(permissions.DELETE)
  async delete(@IdParam() id: number): Promise<void> {
    await this.saleQuotationTemplateService.delete(id);
  }
}
